@@ -8,7 +8,11 @@ namespace fst {
 
 static FstRegisterer<CompactStringFst<StdArc, uint16>>
     CompactStringFst_StdArc_uint16_registerer;
+
 static FstRegisterer<CompactStringFst<LogArc, uint16>>
     CompactStringFst_LogArc_uint16_registerer;
 
+static FstRegisterer<CompactStringFst<Log64Arc, uint16>>
+    CompactStringFst_Log64Arc_uint16_registerer;
+
 }  // namespace fst
