@@ -8,7 +8,11 @@ namespace fst {
 
 static FstRegisterer<CompactStringFst<StdArc, uint8>>
     CompactStringFst_StdArc_uint8_registerer;
+
 static FstRegisterer<CompactStringFst<LogArc, uint8>>
     CompactStringFst_LogArc_uint8_registerer;
 
+static FstRegisterer<CompactStringFst<Log64Arc, uint8>>
+    CompactStringFst_Log64Arc_uint8_registerer;
+
 }  // namespace fst
