@@ -3,8 +3,8 @@
 //
 // Functions and classes to compute the concatenative closure of an FST.
 
-#ifndef FST_LIB_CLOSURE_H_
-#define FST_LIB_CLOSURE_H_
+#ifndef FST_CLOSURE_H_
+#define FST_CLOSURE_H_
 
 #include <algorithm>
 #include <vector>
@@ -30,8 +30,6 @@ namespace fst {
 // where V is the number of states.
 template <class Arc>
 void Closure(MutableFst<Arc> *fst, ClosureType closure_type) {
-  using Label = typename Arc::Label;
-  using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
   const auto props = fst->Properties(kFstProperties, false);
   const auto start = fst->Start();
@@ -133,4 +131,4 @@ using StdClosureFst = ClosureFst<StdArc>;
 
 }  // namespace fst
 
-#endif  // FST_LIB_CLOSURE_H_
+#endif  // FST_CLOSURE_H_
