@@ -3,10 +3,11 @@
 //
 // Regression test for FST classes.
 
-#include "./fst_test.h"
+#include <fst/test/fst_test.h>
 
 #include <utility>
 
+#include <fst/flags.h>
 #include <fst/log.h>
 #include <fst/compact-fst.h>
 #include <fst/const-fst.h>
